#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "_exhaustive_wait4xi.py">
#<abstract4xi abstract = "provides the function _exhaustive_wait4xi"/>
#<refer4xi reference = "connection.py"/>
#<note4xi note = "called only in Windows platform - platform independent"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = ['_exhaustive_wait4xi']
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
import sys
import threading                
import _thread
import socket
import errno
import time
import tempfile
import itertools
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from print4xi import print4xi
from time4xi import time4xi
#"
#/>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "_exhaustive_wait4xi" usage = "
#   connection.py
#       def wait(object_list, timeout=None):
#           waithandle_to_obj = {}
#           ...
#           waithandle_to_obj[o.__index__()] = o
#           ...
#           ready_handles = _exhaustive_wait(waithandle_to_obj.keys(), timeout)
#
#"> 
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
def _exhaustive_wait4xi(handles, timeout):
    '''
    returnn ALL handles which are currently signalled
    '''
    
    # -----------------------------------------------------------------------------------------    
    # (Step 0/4) 
    # -----------------------------------------------------------------------------------------        
    class2print = "_exhaustive_wait4xi"
    method2print = "_exhaustive_wait4xi"

    
    message2print = "{(Step 0A/4)  started" + "; _exhaustive_wait4xi-started: " + str(_exhaustive_wait4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
        
    message2print = "{(Step 0B/4) passed" + "; handles-passed-to-_exhaustive_wait4xi: " + str(handles) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
    # 
     
    message2print = "{(Step 0C/4) passed" + "; timeout-passed-to-_exhaustive_wait4xi: " + str(timeout) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
    # 
                
    # -----------------------------------------------------------------------------------------    
    # (Step 1/4) 
    # -----------------------------------------------------------------------------------------        
    L2xi = list(handles)
    message2print = "{(Step 1/4a) set" + "; L2xi-set-by-_exhaustive_wait4xi to: " + str(L2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #   
            
    ready2xi = []
    message2print = "{(Step 1/4b) set" + "; ready2xi-set-by-_exhaustive_wait4xi to: " + str(ready2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    
    while L2xi:
        # -----------------------------------------------------------------------------------------    
        # (Step 2/4) 
        # -----------------------------------------------------------------------------------------
        res2xi = _winapi.WaitForMultipleObjects(L2xi, False, timeout)
        message2print = "{(Step 2a/4) set" + "; res2xi-set-by-_exhaustive_wait4xi to: " + str(res2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        
        if res2xi == WAIT_TIMEOUT:
            message2print = "{(Step 2b/4) got" + "; res2xi-got-by-_exhaustive_wait4xi: " + str(res2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            
            break
            
        elif WAIT_OBJECT_0 <= res2xi < WAIT_OBJECT_0 + len(L2xi):
            res2xi -= WAIT_OBJECT_0
            message2print = "{(Step 2c/4) set" + "; res2xi-set-by-_exhaustive_wait4xi to: " + str(res2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
        elif WAIT_ABANDONED_0 <= res2xi < WAIT_ABANDONED_0 + len(L2xi):
            res2xi -= WAIT_ABANDONED_0
            message2print = "{(Step 2d/4) set" + "; res2xi-set-by-_exhaustive_wait4xi to: " + str(res2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #   
        else:
            message2print = "{(Step 2z1/4) failed" + "; res2xi-failed-by-_exhaustive_wait4xi: " + str(res2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            raise RuntimeError('(Step 2z2/4) of _exhaustive_wait4xi *** error Should not get here ***')
            
        # -----------------------------------------------------------------------------------------    
        # (Step 3/4) 
        # -----------------------------------------------------------------------------------------     
        ready2xi.append(L2xi[res2xi])
        message2print = "{(Step 3a/4) appended" + "; ready2xi-appended-by-_exhaustive_wait4xi: " + str(ready2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        message2print = "{(Step 3b/4) appended" + "; element-of-L2xi-appended-by-_exhaustive_wait4xi: " + str(L2xi[res2xi])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        L2xi = L2xi[res2xi+1:]
        
        message2print = "{(Step 3c/4) set" + "; L2xi-set-by-_exhaustive_wait4xi to: " + str(L2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        timeout = 0
        message2print = "{(Step 3d/4) set" + "; timeout-set-by-_exhaustive_wait4xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
    # -----------------------------------------------------------------------------------------    
    # (Step 4/4)
    # -----------------------------------------------------------------------------------------     
    message2print = "{(Step 4/4) returning" + "; ready2xi-returning-by-_exhaustive_wait4xi: " + str(ready2xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 
    
    return ready2xi
    
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_exhaustive_wait4xi.txt
#   sense-test-_exhaustive_wait4xi.xml
#   reason-test-_exhaustive_wait4xi.xml
#   ace-test-_exhaustive_wait4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of _exhaustive_wait4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>


